/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package performance;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Collect monitor point timing statistics.
 * 
 * @author <a href="mailto:antelder@apache.org">Ant Elder</a>
 */
public class Monitor {

   static HashMap timers = new HashMap();
   
   public static void start(String s) {
      Timer t = (Timer)timers.get( s );
      if ( t == null ) {
   	     t = new Timer();
         timers.put( s, t );
      } 
      t.start();	
   }

   public static void stop(String s) {
      Timer t = (Timer)timers.get( s );
      t.stop();	
   }

   public static void pause(String s) {
      Timer t = (Timer)timers.get( s );
      if ( t != null ) {
         t.pause();
      }	
   }

   public static void resume(String s) {
      Timer t = (Timer)timers.get( s );
      if ( t != null ) {
         t.resume();	
      }
   }
   
   public static void clear() {
      timers = new HashMap();
   }

   public static HashMap getAvgResults() {
      Timer t;
      String testName;
      HashMap results = new HashMap();
      for (Iterator i = timers.keySet().iterator(); i.hasNext(); ) {
         testName = (String)i.next();
         t = (Timer)timers.get( testName );
         float f1 = t.getAvgTime();
         Float f = new Float( t.getAvgTime() );
         results.put( testName, new Float( t.getAvgTime() ) );
      }
      return results;
   }

   public static void printResults() {
      Timer t; 
      String testName;
      float value;
      ArrayList l = new ArrayList(); 
      for (Iterator i = timers.keySet().iterator(); i.hasNext(); ) {
         l.add( i.next() );
      }

      Collections.sort( l );
      DecimalFormat formatter = new DecimalFormat("#0.00000");

      System.out.println( "\n=== Monitor Results ===\n" );    	
      for (Iterator i = l.iterator(); i.hasNext(); ) {
         testName = (String)i.next();
         t = (Timer)timers.get( testName );
         //value = ((float)Math.round( t.getAvgTime() * 10000 ) / 10000 );     
         System.out.println( pad(testName,40) + 
            " value=" + lpad(formatter.format( t.getAvgTime() ),8) + " msecs" +
            ", samples= " + lpad( ""+t.getIterations(), 8 ) );    	
      }
      System.out.println();    	
    }
        
    public static String pad(String s, int pad) {
       if ( s==null ) s = "";
       StringBuffer sb = new StringBuffer( pad );
       for (int i = 0; i < (pad-s.length()); i++) {
         sb = sb.append( " " );
       }
       return s + sb.toString();
    }

    public static String lpad(String s, int pad) {
       if ( s==null ) s = "";
       StringBuffer sb = new StringBuffer( pad );
       for (int i = 0; i < (pad-s.length()); i++) {
         sb = sb.append( " " );
       }
       return sb.toString() + s;
    }

}

class Timer {

   long start;
   long interval;
   long totalTime;
   int startCount;
   long minimumInterval, errorInterval;
   static final long CLOCK_ACCURACY = 10; // 10 milliseconds
         
   public Timer() {
   	   interval = 0;
   	   totalTime = 0;
   	   startCount = 0;
   	   minimumInterval = Long.MAX_VALUE;
   	   errorInterval = Long.MAX_VALUE;
   }

   public void start() {
   	   interval = 0;
   	   startCount +=1;
   	   totalTime += interval;
   	   start = System.currentTimeMillis();
   }

   public void pause() {
   	   interval += System.currentTimeMillis() - start;
   	   start = 0;
   }

   public void resume() {
   	   start = System.currentTimeMillis();
   }

   public void stop() {
   	   long thisInterval = System.currentTimeMillis() - start;
   	   if ( thisInterval > errorInterval ) {
   	      startCount-=1; // ignore this sample
   	   } else {
   	      interval += thisInterval;
   	      totalTime += thisInterval;
   	      if ( startCount > 1 && minimumInterval > 0 && interval < minimumInterval ) {
   	         minimumInterval = interval;
   	         errorInterval = minimumInterval + CLOCK_ACCURACY;
   	         startCount = 0;
   	         totalTime = 0;
   	      }
   	   }
   }

   public long getInterval() {
   	   return interval;
   }
	
   public float getAvgTime() {
   	   return (float)totalTime / startCount;
   }
	
   public int getIterations() {
   	   return startCount;
   }
	
}