/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package wsdl;

/**
 * Junit test to test out different ways to load WSDL
 * @author Mark Whitlock
 */

import java.io.PrintWriter;
import java.io.StringWriter;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.apache.wsif.WSIFService;
import org.apache.wsif.WSIFServiceFactory;
import util.AddressUtility;
import util.TestUtilities;

import addressbook.wsifservice.AddressBook;
import addressbook.wsiftypes.Address;
import addressbook.wsiftypes.Phone;

public class WsdlLoadingTest extends TestCase {
    String urlWsdl = "http://localhost:8080/wsdl/AddressBook.wsdl";
    String fileWsdl = 
      TestUtilities.getWsdlPath("java\\test\\addressbook\\wsifservice") +
      "AddressBook.wsdl";
    static String server = TestUtilities.getSoapServer().toUpperCase();
      
    static String name1 = "Purdue Boilermaker";
    static Address addr1 = new Address (1, "University Drive",
                                      "West Lafayette", "IN", 47907,
                                      new Phone (765, "494", "4900"));

    public WsdlLoadingTest(String name) { super(name); }
        
	public static void main(String[] args) 
	  { junit.textui.TestRunner.run (suite()); }
	
	public static Test suite()
	  { return new TestSuite(WsdlLoadingTest.class); }
	  
	public void setUp() { TestUtilities.setUpExtensionsAndProviders(); }
	
    public void testUrl() 
    { 
      if (TestUtilities.areWeTesting("remotewsdl")) 
        doit("http://localhost:8080/wsdl/AddressBook.wsdl",null); 
    }
      
    public void testFile() 
    {
      doit(TestUtilities.getWsdlPath("java\\test\\addressbook\\wsifservice") +
           "AddressBook.wsdl",null); 
    }

    public void testBadUrl() 
    { 
      if (TestUtilities.areWeTesting("remotewsdl")) 
        doit("http://localhost:8080/wsdl/AddressBok.wsdl","MalformedURLException"); 
    }
      
    public void testBadFile() 
    {
      doit(TestUtilities.getWsdlPath("java\\test\\addressbook\\wsifservice") +
           "AddressBok.wsdl","MalformedURLException"); 
    }

    public void testImport() 
    { 
      if (TestUtilities.areWeTesting("remotewsdl")) 
        doit("http://localhost:8080/wsdl/ImportingAddressBook.wsdl",null); 
    }
      
    private void doit (String wsdl, String expectedException) 
    {
      try 
      {
      	WSIFServiceFactory factory = WSIFServiceFactory.newInstance();
      	WSIFService service = factory.getService(
          wsdl,
          null,                                           // serviceNS
          null,                                           // serviceName
          "http://wsifservice.addressbook/", // portTypeNS
          "AddressBook");                                 // portTypeName

        AddressBook stub=(AddressBook)service.getStub(server+"Port", AddressBook.class);

        stub.addEntry(name1, addr1);
        Address resp1 = stub.getAddressFromName (name1);
        assertTrue(new AddressUtility(addr1).equals(resp1));
        assertTrue(expectedException==null);
        
  	  } 
  	  catch (Exception e) 
  	  {
  	  	if (expectedException==null) 
  	  	{
   	      System.err.println("WsdlLoadingTest("+wsdl+") caught exception " + e);
	      e.printStackTrace();
	      assertTrue(false);
  	  	}
  	  	else
  	  	{
	   	  StringWriter sw=new StringWriter();
  	      PrintWriter pw=new PrintWriter(sw);
     	  e.printStackTrace(pw);
  	      String stack=sw.getBuffer().toString();
  	      assertTrue(stack.indexOf(expectedException)>0);
  	  	}
  	  }
    }
    
}
